#!/bin/bash
#   ---------------------------------------------------------------------- copyright and license ---
#
#   file: bin/build.m4
#
#   Copyright © 2019 Van de Bugger.
#
#   This file is a part of S4lt.
#
#   S4lt is free software: you can redistribute it and/or modify it under the terms of the GNU
#   General Public License as published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   S4lt is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
#   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along with S4lt.  If not, see
#   <http://www.gnu.org/licenses/>.
#
#   SPDX-License-Identifier: GPL-3.0-or-later
#
#   ---------------------------------------------------------------------- copyright and license ---

#   This is a (thin?) wrapper around `autoreconf && configure && make`. It does few things:
#
#       1.  It runs `autoreconf` if `configure` does not exist or outdated.
#       2.  It creates `_build` directory, and runs other actions from within it.
#       3.  It implements `veryclean` (pseudo)target.
#
#   The last item is quite important because I do not like how it is implemented in `Makefile` —
#   it builds a lot of stuff just to delete everything.

set -Eeo pipefail

# Let's colorize result output:
if [[ -t 1 ]]; then
    gre='\e[32m'
    red='\e[31m'
    yel='\e[33m'
    rst='\e[0m'
else
    gre=''
    red=''
    yel=''
    rst=''
fi

status=0
trap '[[ "$status" != "0" ]] || echo -e "${gre}+++++ SUCCESS +++++${rst}";'                EXIT
trap 'status=1;                 echo -e "${red}----- FAILURE -----${rst}";'                ERR
trap 'status=2; echo;           echo -e "${yel}***** ABORTED *****${rst}"; exit $status;'  INT

script=$( basename "$0" )
curdir=$( pwd )
bindir=$( dirname "$0" )
srcdir=$( cd "$bindir/.." && pwd )
builddir="$curdir/_build"
installdir="$builddir/_install"

conf_opts=()
make_opts=()
make_vars=()
goals=()

# Last performed action, either clean, very-clean, configure, or make.
last_action=
prep=

banner() {
    printf '%s\n' "[b] ----- $* -----"
}

run() {
    printf '%s\n' "$*"
    "$@"
}

contains() {
    local pattern item
    pattern="$1"
    shift 1
    for item in "$@"; do
        case "$item" in
            $pattern )
                return 0
                ;;
            * )
                :
                ;;
        esac
    done
    return 1
}

match() {
    local item pattern
    item="$1"
    shift 1
    for pattern in "$@"; do
        case "$item" in
            $pattern )
                return 0
                ;;
            * )
                :
                ;;
        esac
    done
    return 1
}

help() {
    #     12345678901234567890123456789012345678901234567890123456789012345678901234567890
    echo "$script makes developer tasks a bit easier: it creates build directory"
    echo "and runs required autoreconf/configure/make with specified and/or implied"
    echo "options."
    echo ""
    echo "Usage:"
    echo "    $script [CONFIGURE-OPTION]... [MAKE-OPTION]... [TARGET]..."
    echo "        Build specified or impled targets by running autoreconf/configure/make."
    echo ""
    echo "        Options starts with --with, --without, --enable and --disable are"
    echo "        passed to configure, other options and targets are passed to make."
    echo ""
    echo "        Target very-clear (or just vc) is treated specially: instead of passing"
    echo "        it to make, $script clears build directory and all intermediate files"
    echo "        generated by either autoreconf or configure."
    echo ""
    echo "    $script --help"
    echo "        Print $script help and exit."
    echo ""
    echo "    $script --help-configure"
    echo "        Run configure with --help option."
    echo ""
    echo "    $script help"
    echo "        Show make convenience targets."
    echo ""
    exit 0
}

#   I want to (1) show actions, and (2) minimize output. So, show the rm command only if file
#   actually exists and we are going to remove it, skip echo for non-existent files.
_remove() {
    for file in "$@"; do
        if [[ -f "$file" ]]; then
            run rm "$file"
        elif [[ -d "$file" ]]; then
            run rm -r "$file"
        fi
    done
}

#   Do preparation: create build directory and make it current.
do_prep() {
    [[ "$prep" != "done" ]] || return 0
    local banner="banner 'prep'; banner=':'"    # Show banner only once.
    [[ -d "$builddir" ]]          || { eval $banner; run mkdir -p "$builddir"; }
    [[ "$builddir" = "$curdir" ]] || { eval $banner; run cd "$builddir";       }
    prep=done
}

do_clean() {
    case "$last_action" in
        *clean )
            # Avoid repeating cleaning.
            return 0
            ;;
    esac
    banner 'clean'
    run "$srcdir/bin/clean-dir.sh" "--exclude=config.*" .
    last_action=clean
}

#   Similar to make's maintainer-clean, but does not rebuild configure and Makefile to delete them.
do_very_clean() {
    case "$last_action" in
        very-clean)
            # Avoid repeating cleaning.
            return 0
            ;;
    esac
    banner 'very-clean'
    run "$srcdir/bin/clean-dir.sh" .
    _remove "$srcdir"/.aux
    _remove "$srcdir"/aclocal.m4
    _remove "$srcdir"/config{.cache,.h.in,.log,.status,.status.lineno}
    _remove "$srcdir"/configure{,.lineno}
    _remove "$srcdir"/Makefile.in
    last_action=very-clean
}

do_autoreconf() {
    banner 'autoreconf'
    _remove Makefile      # Just in case.
    run autoreconf -i "$srcdir"
    last_action=autoreconf
}

do_configure() {
    banner 'configure'
    run "$srcdir/configure" --enable-maintainer-mode "${conf_opts[@]}"
    if contains "--help" "${conf_opts[@]}"; then
        exit 0
    fi
    # Clear configure options to avoid run it for the next time.
    conf_opts=()
    last_action=configure
}

do_make() {
    local rc=0
    banner 'make'
    run make "${make_opts[@]}" "${make_vars[@]}" "${goals[@]}"
    goals=()
    last_action=make
}

# Build goals accumulated in goal array, and clear the array.
# If goals array is empty, do nothing.
build() {
    if [[ "${#goals[@]}" -eq 0 ]]; then
        # Nothing to build.
        return 0
    fi
    # Makefile generated by autoconf tracks changes in configure.ac and rebuilds configure and
    # Makefile. But it is not enouh, because other files are not accurately rebuilt. For example,
    # changing version and/or release in configure.ac does cause rebuilding configure and makefile,
    # but it does *not* cause rebuilding spec file. Thus, built rpm file will have invalid name.
    # Problem could be fixed by adding configure.ac and makefile.am to prerequisites for *each*
    # target file in the Makefile, but (1) it is hardly possible due to lots of targets implicitly
    # added by automake, and (2) manual adding a prerequicite to each target is error-prone.
    # Deleting everything if configure.ac is changed may be not very efficient, but quite
    # effective way to avoid invalid builds.
    if [[ "$srcdir/configure.ac" -nt "$srcdir/configure" \
       || "$srcdir/RELEASE.txt"  -nt "$srcdir/configure" ]]; then
        do_very_clean
    fi
    if [[ ! -e "$srcdir/configure" ]]; then
        do_autoreconf
    fi
    # TODO: Can I move it up, before autoreconf?
    # The reasoning above is applicable to Makefile.am too.
    # If Makefile.am is changed, clean the build directory just in case.
    # (clean does not delete configure files.)
    if [[ "$srcdir/Makefile.am" -nt "Makefile" ]]; then
        do_clean
    fi
    # Run configure if there is no Makefile or we have configure options.
    if [[ ! -e "Makefile" || "${#conf_opts[@]}" -ne 0 ]]; then
        do_configure
    fi
    do_make
}

# Sets DESTDIR temporary and builds the specified goal.
# DESTDIR can't be set permanently because it badly interferes with distcheck automake goal:
# make creates too long path like "/PATH/_build/install/PATH/_build/blankom4t/_inst/share/aclocal".
# See <https://debbugs.gnu.org/cgi/bugreport.cgi?bug=34166>.
install() {
    DESTDIR="$installdir" build
}

# Parse command line. Collect separately options, variable assignments, and goals.
todo=()
for arg in "$@"; do
    case "$arg" in
        --help )
            help
            ;;
        --help-conf | --help-configure )
            conf_opts+=( "--help" )
            ;;
        --with-* | --without-* | --enable-* | --disable-* | --prefix=* | --*dir=* )
            conf_opts+=( "$arg" )
            ;;
        -* )
            make_opts+=( "$arg" )
            ;;
        *=* )
            make_vars+=( "$arg" )
            ;;
        * )
            todo+=( "$arg" )
            ;;
    esac
done
# If no goals specified explicitly, assume "all" goal.
if [[ "${#todo[@]}" -eq 0 ]]; then
    todo=( "all" )
fi

do_prep

for goal in "${todo[@]}"; do
    case "$goal" in
        vc | veryclean | very-clean )
            build
            do_very_clean
            ;;
        install | install-strip | uninstall )
            build
            goals+=( "$goal" )
            install
            ;;
        * )
            goals+=( "$goal" )      # Remember the goal, will build all the goals later.
            ;;
    esac
done
build

exit 0

# end of file #
