#!/bin/bash

#   ---------------------------------------------------------------------- copyright and license ---
#
#   file: bin/check-s4lt-autom4te.sh
#
#   Copyright © 2019 Van de Bugger.
#
#   This file is a part of S4lt.
#
#   S4lt is free software: you can redistribute it and/or modify it under the terms of the GNU
#   General Public License as published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   S4lt is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
#   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along with S4lt.  If not, see
#   <http://www.gnu.org/licenses/>.
#
#   SPDX-License-Identifier: GPL-3.0-or-later
#
#   ---------------------------------------------------------------------- copyright and license ---

eval $PROLOGUE

[[ $# -lt 1 ]] && die "No arguments."
[[ $# -gt 1 ]] && die "Too many arguments."
[[ -n ${AUTOM4TE} ]] || skip "AUTOM4TE is not set."
[[ -n ${DIFF}     ]] || skip "DIFF is not set."
[[ -n ${PERL}     ]] || skip "PERL is not set."

name="$1"
# A test can be either positive, negative, or documentation.
type=pos
[[ "$name" =~ .*\.doc$ ]] && type=doc
[[ "$name" =~ .*\.bad$ ]] && type=neg

input="$SRCDIR/test/$name.input"            # Input file.
expected="$SRCDIR/test/$name.output"        # Expected output file.
actual="$name.output"                       # Actual output file.
s4lt="$SRCDIR/src/s4lt.m4"

[[ -e "$input"    ]] || die "'$input': file does not exist."
[[ -f "$input"    ]] || die "'$input': is not a file."
[[ -e "$expected" ]] || die "'$expected': file does not exist."
[[ -f "$expected" ]] || die "'$expected': is not a file."

case "$type" in
    pos )
        #   Positive test.
        #   *   echo: Autoconf produces no output until `AC_INIT`, so we have to explicitly call
        #       `m4_divert` to have output.
        #   *   echo: Inject actual file name to output file as the first line comment.
        #   *   perl: `white-space.output` has some hardly visible whitespace characters. Let's
        #        make HT and CR visible.
        echo "m4_divert[]# file: $actual #" | \
            $AUTOM4TE -l autoconf "$s4lt" - "$input" | \
            $PERL -p -e "s|\\t|\\\\t|g; s|\\r|\\\\r|g;" > "$actual"
        ;;
    doc )
        {
            echo 'm4_define([_indent], [m4_bpatsubst([[$2]], [^], [$1])])'
            echo 'm4_define([EG], [_indent([    ⇐ ], [{$1}])'
            echo ''
            echo '_indent([    ⇒ ], _m4_expand([{$1}]))])'
            echo "m4_divert[]<!-- # file: $actual # -->"
        } | \
            $AUTOM4TE -l autoconf "$s4lt" - "$input" > "$actual"
        ;;
    neg )
        #   Negative test requires special treatment.
        #   *   `autom4te` is expected to fail, i. e. return non-zero status.
        #   *   I do not need stdout, so there is no need in calling `m4_divert`.
        #   *   I need stderr instead of stdout.
        #   *
        $AUTOM4TE -l autoconf "$s4lt" "$input" 2> "$actual.tmp" && rc=$? || rc=$?
        [[ "$rc" -eq 1 ]] || fail "autom4te expected to fail."
        #   Output file contains error messages which include file names. So, output is
        #   location-dependent: if someone run tests in different location, output files will differ.
        #   Let's drop paths to make output location-independent. Also drop `s4lt.m4` line numbers: I
        #   do not want test fail just because `s4lt.m4` file changed.
        $PERL -p \
            -e "s|^\Q$input\E:(\d+):|*/$name.input:\$1:|g or " \
            -e "s|^\S*?/s4lt\.m4:\d+:|*/s4lt.m4:*:|g or " \
            -e "s|^autom4te: \S*?/m4 failed|autom4te: */m4 failed|g;" \
            "$actual.tmp" > "$actual"
        ;;
    * )
        die "oops: unknown test type: '$type'"
        ;;
esac
$DIFF -u "$expected" "$actual"

# end of file #
