#!/bin/bash
#   ---------------------------------------------------------------------- copyright and license ---
#
#   file: bin/clean-dir.sh
#
#   Copyright © 2019 Van de Bugger.
#
#   This file is a part of S4lt.
#
#   S4lt is free software: you can redistribute it and/or modify it under the terms of the GNU
#   General Public License as published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   S4lt is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
#   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along with S4lt.  If not, see
#   <http://www.gnu.org/licenses/>.
#
#   SPDX-License-Identifier: GPL-3.0-or-later
#
#   ---------------------------------------------------------------------- copyright and license ---

set -eo pipefail

version='0.1'

die() {
    echo "$( basename "$0" ): $*" >&2
    exit 99
}

version() {
    echo "$script v$version."
}

help() {
    #     12345678901234567890123456789012345678901234567890123456789012345678901234567890
    echo "SYNOPSIS"
    echo "    $script [OPTION...] [--] [DIR]..."
    echo ""
    echo "DESCRIPTION"
    echo "    The script cleans specified directories -- removes all the files and"
    echo "    subdirs (including hidden ones) in these directories recursively, but does"
    echo "    not remove directories itself. Some files/subdirs may be excluded from"
    echo "    removing by using --exclude option."
    echo ""
    echo "OPTIONS"
    echo "    --help"
    echo "        Show this help message and exit."
    echo ""
    echo "    -n, --dry-run"
    echo "        Show commands but do not execute them."
    echo ""
    echo "    --verbose"
    echo "        Show names of files being deleted."
    echo ""
    echo "    --version"
    echo "        Show version and exit."
    echo ""
    echo "    -x, --exclude PATTERN"
    echo "        Do not remove entries match this pattern. (Do not forget to protect"
    echo "        the pattern from shell!)"
    echo ""
    echo "EXAMPLE"
    echo "    $script -x 'config.*' _build"
    echo ""
}

match() {
    local item pattern
    item="$1"
    shift 1
    for pattern in "$@"; do
        case "$item" in
            $pattern )
                return 0
                ;;
            * )
                :
                ;;
        esac
    done
    return 1
}

#
#   Parse command line.
#

exclude=( "." ".." )    # No arrays in dash.
run=""
verbose=""

eval set -- $( \
    /bin/getopt -n "$script" \
        --options "nx:v" \
        --long "help,version,exclude:,dry-run,verbose" \
        -- "$@" \
)
while true; do
    case "$1" in
        --help )
            help
            exit 0
            ;;
        -n | --dry-run )
            run="echo"
            shift 1
            ;;
        -v | --verbose )
            verbose=v
            shift 1
            ;;
        --version )
            version
            exit 0
            ;;
        -x | --exclude )
            exclude+=( "$2" )
            shift 2
            ;;
        -- )
            shift 1
            break
            ;;
        * )
            die "oops"
            ;;
    esac
done

#
#   Do the work.
#

if [[ $# -eq 0 ]]; then
    die "No dirs specified -- nothing to do."
fi

for dir in "$@"; do
    $run /bin/chmod -R u+w "$dir"
    /bin/ls -a -1 "$dir" | \
    while read file; do
        if match "$file" "${exclude[@]}"; then
            if [[ -n "$verbose" ]]; then
                echo "skipped '$dir/$file'"
            fi
        else
            $run /bin/rm -rf$verbose -- "$dir/$file"
        fi
    done
done

exit 0

# end of file #
