#!/bin/bash
#   ---------------------------------------------------------------------- copyright and license ---
#
#   file: bin/fossil-uv-add.sh
#
#   Copyright © 2019 Van de Bugger.
#
#   This file is a part of S4lt.
#
#   S4lt is free software: you can redistribute it and/or modify it under the terms of the GNU
#   General Public License as published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   S4lt is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
#   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along with S4lt.  If not, see
#   <http://www.gnu.org/licenses/>.
#
#   SPDX-License-Identifier: GPL-3.0-or-later
#
#   ---------------------------------------------------------------------- copyright and license ---

set -eo pipefail

. $( dirname "$0" )/fossil.sh

: ${CMP:=/usr/bin/cmp}
: ${GREP:=/usr/bin/grep}

#
#   Functions.
#

function help() {
    cat <<EOT
NAME
    $script — add an unversion file to the fossil repository, if the file
    content changed.

SYNOPSIS
    $script [OPTION...] [--] FILE

DESCRIPTION
    "fossil uv add" command adds unversioned files to the fossil repository
    unconditionally, even if exactly the same file is already present in the
    repository. $script adds a file only if the file content differs.

OPTIONS
    --as FILE
        Name of unversioned file in the fossil repository.

    -h, --help
        Show this help message and exit.

    -n, --dry-run
        Do not add them actually (but show files selected for removal if
        verbosity level is high enough).

    --quiet
        Descrease verbosity level.

    -R, --repository REPO
        Path to fossil repository.

    --verbose
        Increase verbosity level.

ARGUMENTS

    FILE
        Disk file to add to the repository.

ENVIRONMENT

    FOSSIL
        Fossil executable to run. If not set, "fossil" is used by default.

EXAMPLES

    Remove all the files in "site" directory exept those which are in
    "site/trunk" directory:

        $script -x 'site/trunk/*' 'site/*'
EOT
};

#   Returns true (0) if remote file exists, and false otherwise.
#   Usage:  exist REMOTE
#   where
#       REMOTE — name of unversioned file in the fossil repository.
function exist() {
    local remote="$1"
    #   I do not use pipes because it is hardly possible to check status of the non-last elements
    #   of pipe.
    make_tmpfile
    local list="$tmpfile"
    fsl ls > "$list"
    run 1 $GREP -Fq "$remote" "$list"
};

#   Returns true (0) if local and remote files have exactly the same content, and false otherwise.
#   Usage:  the_same LOCAL REMOTE
#   where
#       LOCAL — disk file name.
#       REMOTE — name of unversioned file in the fossil repository.
function the_same() {
    local local="$1" remote="$2"
    make_tmpfile
    local content="$tmpfile"
    fsl cat "$remote" > "$content"
    run 1 $CMP -s "$local" "$content"
};

#   Adds file to fossil repository.
#   Usage:  add LOCAL REMOTE
#   where
#       LOCAL — disk file name.
#       REMOTE — name of unversioned file in the fossil repository.
function add() {
    local local="$1" remote="$2"
    if exist "$remote" && the_same "$local" "$remote"; then
        : # Remote file exists and is the same as local file.
        say 1 "File $remote is up-to-date."
    else
        say 1 "Adding file $remote…"
        fsl ${dry_run[@]} add "$local" --as "$remote"
    fi
};

#
#   Parse command line.
#

as=

eval set -- $( \
    /usr/bin/getopt -n "$script" \
        --options "hnqR:v" \
        --long "as:,dry-run,help,quiet,repository:,verbose" \
        -- "$@" \
)
while true; do
    case "$1" in
        --as )
            as="$2"
            shift 2
            ;;
        -h | --help )
            help
            ;;
        -n | --dry-run )
            dry_run=( ":" )
            shift 1
            ;;
        -q | --quiet )
            verbosity=$(( verbosity - 1 ))
            shift 1
            ;;
        -R | --repository )
            repository=( "$1" "$2" )
            shift 2
            ;;
        -v | --verbose )
            verbosity=$(( verbosity + 1 ))
            shift 1
            ;;
        -- )
            shift 1
            break
            ;;
        * )
            die "oops"
            ;;
    esac
done

[[ $# -gt 0 ]] || die "No files to add specified."
[[ $# -eq 1 ]] || die "Adding multiple files is not yet supported."

local="$1"          # Disk file name.
if [[ -z "$as" ]]; then
    remote="$1"     # Name of unversioned file in the fossil repository.
else
    remote="$as"
fi

#
#   Do the work.
#

add "$local" "$remote"

# end of file #
