#!/bin/bash
#   ---------------------------------------------------------------------- copyright and license ---
#
#   file: bin/fossil-uv-rm.sh
#
#   Copyright © 2019 Van de Bugger.
#
#   This file is a part of S4lt.
#
#   S4lt is free software: you can redistribute it and/or modify it under the terms of the GNU
#   General Public License as published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   S4lt is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
#   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along with S4lt.  If not, see
#   <http://www.gnu.org/licenses/>.
#
#   SPDX-License-Identifier: GPL-3.0-or-later
#
#   ---------------------------------------------------------------------- copyright and license ---

set -eo pipefail

. $( dirname "$0" )/fossil.sh

#
#   Functions.
#

function help() {
    cat <<EOT
NAME
    $script — remove unversioned files from the fossil repository.

SYNOPSIS
    $script [OPTION...] [--] PATTERN...

DESCRIPTION
    "fossil uv rm" command removes unversioned files from the fossil
    repository, but it requires exact file names, patterns are not supported.
    $script workarounds this fossil drawback.

OPTIONS
    -h, --help
        Show this help message and exit.

    -n, --dry-run
        Do not remove files actually (but show files selected for removal if
        verbosity level is high enough).

    --quiet
        Descrease verbosity level.

    -R, --repository REPO
        Path to fossil repository.

    --verbose
        Increase verbosity level.

    -x, --exclude PATTERN
        Do not remove files which match this pattern. Protect the pattern from
        expansion by the shell!

ARGUMENTS

    PATTERN
        Remove unversioned files which match this pattern. Protect the pattern
        from expansion by the shell!

ENVIRONMENT

    FOSSIL
        Fossil executable to run. If not set, "fossil" is used by default.

EXAMPLES

    Remove all the files in "site" directory exept those which are in
    "site/trunk" directory:

        $script -x 'site/trunk/*' 'site/*'
EOT
};

#
#   Parse command line.
#

exclude=()      # List of files to exclude from removal.

eval set -- $( \
    /usr/bin/getopt -n "$script" \
        --options "hnqR:vx:" \
        --long "dry-run,exclude:,help,quiet,repository:,verbose" \
        -- "$@" \
)
while true; do
    case "$1" in
        -h | --help )
            help
            ;;
        -n | --dry-run )
            dry_run=( ":" )
            shift 1
            ;;
        -q | --quiet )
            verbosity=$(( verbosity - 1 ))
            shift 1
            ;;
        -R | --repository )
            repository=( "$1" "$2" )
            shift 2
            ;;
        -v | --verbose )
            verbosity=$(( verbosity + 1 ))
            shift 1
            ;;
        -x | --exclude )
            exclude+=( "$2" )
            shift 2
            ;;
        -- )
            shift 1
            break
            ;;
        * )
            die "oops"
            ;;
    esac
done

[[ $# -gt 0 ]] || die "No files to remove specified."

#
#   Do the work.
#

total=0         # Total number of uv files.
skipped=0       # Number of skipped uv files.
to_remove=()    # List of uv files to remove.

#   Get complete list of unversioned files:
make_tmpfile
list="$tmpfile"
fsl ls > "$list"

#   Filter the list, collect names of files to remove:
while read file; do
    total=$(( total + 1 ))
    if match "$file" "$@" && ! match "$file" "${exclude[@]}"; then
        say 3 "To remove: '$file'."
        to_remove+=( "$file" )
    else
        say 3 "Skipped  : '$file'."
        skipped=$(( skipped + 1 ))
    fi
done < "$list"
say 1 "$total file(s) found - $skipped file(s) skipped = ${#to_remove[@]} file(s) to remove."

#   Remove files:
if [[ ${#to_remove[@]} -gt 0 ]]; then
    say 1 "Removing file(s) ${to_remove[@]}…"
    fsl "${dry_run[@]}" rm "${to_remove[@]}"
fi

# end of file #
