#   ---------------------------------------------------------------------- copyright and license ---
#
#   file: bin/fossil-uv-rm.sh
#
#   Copyright © 2019 Van de Bugger.
#
#   This file is a part of S4lt.
#
#   S4lt is free software: you can redistribute it and/or modify it under the terms of the GNU
#   General Public License as published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   S4lt is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
#   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along with S4lt.  If not, see
#   <http://www.gnu.org/licenses/>.
#
#   SPDX-License-Identifier: GPL-3.0-or-later
#
#   ---------------------------------------------------------------------- copyright and license ---

: ${FOSSIL:=/usr/bin/fossil}

script="${0##*/}"
verbosity=1     # Verbosity level.
dry_run=()      # If (:), do not remove files actually.
repository=()   # Fossil -R/--repository option with argument.
tmpfiles=()     # List of temporary files creted by the script.

#   Prints messages and exits with status 99.
#   Usage: die LEVEL MESSAGE...
#   where
#       LEVEL —
#       MESSAGE — message to print. Every message prepended with script name and prointed on a
#           separate line.
function say() {
    local level="$1"
    shift 1
    if [[ $verbosity -ge $level ]]; then
        for line in "$@"; do
            echo "$script: $line" >&2
        done
    fi
};

#   Prints error messages and exits with status 99.
#   Usage: die MESSAGE...
#   where
#       MESSAGE — message to print. Every message prepended with script name and prointed on a
#           separate line.
function die() {
    for line in "$@"; do
        echo "$script: $line" >&2
    done
    exit 99
};

function cat() {
    local cat=/usr/bin/cat
    if [[ -t 1 ]]; then
        cat="${PAGER:-/usr/bin/less}"
    fi
    $cat
    exit 0
};

#   Returns true (0) if ITEM matches at least one PATTERN, and false otherwise.
#   Usage:  match ITEM PATTERN...
function match() {
    local item pattern
    item="$1"
    shift 1
    for pattern in "$@"; do
        case "$item" in
            $pattern )
                return 0
                ;;
            * )
                :
                ;;
        esac
    done
    return 1
};

#   Runs external program, dies if program exit status exceeds MAX.
#   Usage:  run MAX COMMAND ARGUMENT...
#   where
#       MAX — maximum value of exit status which does not cause `die`.
#       COMMAND ARGUMENT... — a command to run.
#   Many programs return status 1 as a "false" result which is not an error. For example, `grep`
#   returns status 0 if search was successful, 1 if search was unsuccessful, and > 1 in case of
#   error (e. g. unknown option or non-existent file). I want all `grep` errors are reported
#   with `die`, but unsuccessful search should be reported to the caller:
#       if run 1 grep ...; then ...; else ...; fi
function run() {
    local max="$1" rc=0
    shift 1
    if [[ $verbosity -ge 2 ]]; then
        echo "$@" >&2
    fi
    "$@" || rc=$?
    [[ $rc -le $max ]] || die "$1 exited with non-zero status $rc."
    say 3 "$1 exited with status $rc."
    return $rc
};

#   Make new temporary file.
#   Usage:  make_tmpfile
#   Name of created file is saved in `tmpfile` variable.
function make_tmpfile() {
    tmpfile=$( /usr/bin/mktemp )
    tmpfiles+=( "$tmpfile" )
};

#   Remove all temporary files, if any.
function remove_tmpfiles() {
    if [[ ${#tmpfiles[@]} -gt 0 ]]; then
        local verbosity=$(( verbosity - 1 ))
        run 0 /usr/bin/rm -f "${tmpfiles[@]}"
    fi
};

#   Remove all temporary files at exit.
trap remove_tmpfiles EXIT

#   Usage:  fsl [:] COMMAND ARGUMENT...
#   where
#       COMMAND — fossil uv subcommand, like `add`, `ls`, etc.
#       ARGUMENT... — subcommand options and arguments.
#   If `:` is presented, fossil command is echoed but not executed.
function fsl() {
    local dry_run=()
    if [[ $1 == : ]]; then
        dry_run=( $1 )
        shift 1
    fi
    run 0 ${dry_run[@]} $FOSSIL uv "${repository[@]}" "$@"
};

# end of file #
