#   ---------------------------------------------------------------------- copyright and license ---
#
#   file: mk/vdb.mk
#
#   Copyright © 2019 Van de Bugger.
#
#   This file is a part of S4lt.
#
#   S4lt is free software: you can redistribute it and/or modify it under the terms of the GNU
#   General Public License as published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   S4lt is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
#   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along with S4lt.  If not, see
#   <http://www.gnu.org/licenses/>.
#
#   SPDX-License-Identifier: GPL-3.0-or-later
#
#   ---------------------------------------------------------------------- copyright and license ---

#
#   Shorter names for frequently used entities.
#

name := @PACKAGE_TARNAME@
ver  := @PACKAGE_VERSION@
rel  := @PACKAGE_RELEASE@
if enable_rpm
    dist := @RPM_DIST@
    arch := @RPM_ARCH@
endif

#
#   Useful variables.
#

hash  := \#
null  :=
space := $(null) $(null)
pipefail := if test -n "$$BASH_VERSION"; then set -o pipefail; fi
    #   Pipe (`|`) is a problem: If the last command of a pipe returns zero status, make thinks
    #   everything is ok, while actually some commands may fail. `pipefail` option helps, but it is
    #   a bashism option. Let's use it if available.
prologue = @printf "%s\\n" "[$(MAKELEVEL)] ----- $@ ----- "
    #   Use this variable in the beginning of each non-empty recipe.

#
#   Various lists.
#

All =
    #   List of targets to make by `make All`. It includes optional targets like tgz and html
    #   (which are not made by `make all`), but does not include tests.
EXPORT = @EXPORT@
    #   List of environment variables to export from makefile.
UNEXPORT =
    #   List of environment variables to unexport from makefile.
PHONY =
    #   List of phony targets. Automake complains if a target (including special target `.PHONY`)
    #   appears more than once. Let us avoid the warning by collecting phony target names in this
    #   variable.
SPATH = bin src test
    #   List of source subdirectories. `VPATH` value is constructed from this list.
check_SCRIPTS =
    #   List of scripts which required for testing.
check_PROGRAMS =
    #   List of programs which should be built for testing.
INTFILES =
    #   List of intermediate files. Intemediate files are generated files which don't go to output.
    #   Intermediate files are subject to delete by `make mostlyclean`.
INTDIRS =
    #   List of intermediate (temporary) directories. Intermediate directories are subject to
    #   delete by `make mostlyclean`.
OUTFILES = $(PROGRAMS) $(SCRIPTS) $(DATA) $(MANS)
    #   List of output files. Output files are subject to delete by `make clean`.
OUTDIRS = _install
    #   List of output directories. Output directories are subject to delete by `make clean`.
    #   build.sh sets DESTDIR variable to "$builddir/_install". I want to delete it by `make
    #   clean`.But it is too dangerous to declare `$(DESTDIR)` as output directory. In another
    #   environment (e. g. if make is run manually) `DESTDIR` can be "/" or "/usr/local". So let me
    #   just use "_install". It will work for `build.sh`, and looks relatively harmless in other
    #   circumstances.
    #   TODO: Isn't it an intermediate dir?
HELP =
    #   List of project-specific targets and their help messages, e. g.:
    #       HELP  += All  "make everything (incl. optional targets) but do not run tests"
    #       HELP  += none "make nothing"
TEST_EXTENSIONS = .test
    #   List of test extensions.
TESTS =
    #   List of tests.
VPATH = @srcdir@:$(subst $(space),:,$(addprefix @srcdir@/, $(SPATH)))

#
#  GNU Make directives.
#

.PHONY           : $(PHONY)
.SECONDARY       :
.PRECIOUS        : test-suite.log
    #   Do not delete test-suite.log if make is interrupted.
.DELETE_ON_ERROR :
    #   Delete target if recipe fails.

#
#   Automake directories.
#

distdir     = $(name)-$(ver)-$(rel)
appdir      = $(datadir)/applications
unitdir     = $(exec_prefix)/lib/systemd/system
userunitdir = $(exec_prefix)/lib/systemd/user
aclocaldir  = $(datadir)/aclocal
licensedir  = $(datadir)/licenses/$(name)

#
#   Environment.
#

SRCDIR   = $(srcdir)
PROLOGUE = set -e; $(pipefail); . ./environment.sh; . $$SRCDIR/bin/prologue.sh;
    #   This will be executed in the very beginning of test scripts.
EXPORT += SRCDIR PROLOGUE
export $(EXPORT)
    #   `EXPORT` must not be empty, or `make` will export everything!

# GNOME in Fedora 20 started to set bunch of LC_ environment variables. It breaks many tests.
# Let us unset them.
UNEXPORT += \
    LC_ALL              \
    LC_ADDRESS          \
    LC_COLLATE          \
    LC_CTYPE            \
    LC_IDENTIFICATION   \
    LC_MESSAGES         \
    LC_MEASUREMENT      \
    LC_MONETARY         \
    LC_NAME             \
    LC_NUMERIC          \
    LC_PAPER            \
    LC_TELEPHONE        \
    LC_TIME             \
    $(null)
unexport $(UNEXPORT)

#
#   All and none targets.
#

# `make all` does not build rpms and html. Let us add target which really builds all, but does not
# run tests.
HELP  += All  "make everything (incl. optional targets) but do not run tests"
HELP  += none "make nothing"
PHONY += All none
All   += all
All  : $(All)
none :

#
#   Help target.
#

PHONY += help
help :
	$(prologue)
	@echo "Project-specific convenience targets:"
	@echo
	@printf '    %-16s %s\n' $(HELP)
	@echo
	@echo "GNU Automake targets (e. g. all, check, dist, distcheck, etc) also work."
	@echo

#
#   Cleaning stuff.
#

#   GNU Coding Standards requires 4 clean targets: mostlyclean, clean, distclean, maintainer-clean.

# mostlyclean is a bit weaker version of clean: it may leave some files wich require lot of time to
# rebuilt (and likely not need to be rebuilt). However, I don't have such files, so my mostlyclean
# is (mostly?) identical to clean.
# BTW, automake's mostlyclean deletes test log and result files.
HELP  += mc "= mostlyclean = delete intermediate files and dirs"
PHONY += mc
mc : mostlyclean
MOSTLYCLEANFILES = $(INTFILES)
MOSTLYCLEANDIRS  = $(INTDIRS)
# Automake takes care about deleting files listed in MOSTLYCLEANFILES.
# But MOSTLYCLEANDIRS is my extension, so I have to handle it:
PHONY += mostlyclean-local
mostlyclean-local :
	$(prologue)
	$(if $(MOSTLYCLEANDIRS), rm -rf $(MOSTLYCLEANDIRS))

# clean deletes all the generated files but not results of ./configure (Makefile, config.*,
# configure.*). After clean, make should work (running configure is not required).
HELP  += cc "= clean = mostlyclean + delete output files and dirs"
PHONY += cc
cc : clean
CLEANFILES = $(OUTFILES)
CLEANDIRS  = $(OUTDIRS)
# Again, CLEANDIRS is my extension:
PHONY += clean-local
clean-local :
	$(prologue)
	$(if $(CLEANDIRS), rm -rf $(CLEANDIRS))

# distclean should delete results of ./configure, but do not touch ./configure itself.
# After distclean directory should be the same as after extracting distribution tarball (and so,
# user have to run ./configure again).
HELP  += dc "= distclean, clean + delete files made by configure"
PHONY += dc
dc : distclean
DISTCLEANFILES = $(srcdir)/config.log
    # configure, being run from the source directory, aborts but leaves config.log file.
    # Let us delete it, if any.
DISTCLEANDIRS  = autom4te.cache
    # autom4te.cache directory is not built by configure, but it is not a part of distribution.
    # Also, autom4te documentation says it can be safely deleted at any time, so let me delete it.
PHONY += distclean-local
distclean-local :
	$(prologue)
	$(if $(DISTCLEANDIRS), rm -rf $(DISTCLEANDIRS))
	@: # In case of out-of-source build, make leaves $(builddir)/src empty directory. It looks
	@: # it could be deleted here, but at this time it is not yet empty.

# Both GNU Coding Standards and GNU Automake are not very clear in defining maintainer-clean
# target. It should delete everything which can be reconstructed (probably, with special tools),
# but should not delete configure (which can be reconstructed with autoreconf). Anyway, I do not
# use it. Instead I use veryclean target implemented by build.sh which really deletes everything
# generated.
MAINTAINERCLEANFILES =
MAINTAINERCLEANDIRS  =
PHONY += maintainer-clean-local
maintainer-clean-local :
	$(prologue)
	$(if $(MAINTAINERCLEANDIRS), rm -rf $(MAINTAINERCLEANDIRS))

#
#   Manifest.
#

manifest := $(shell sed -e 's/$(hash).*//' $(srcdir)/MANIFEST.txt)
    #   Files to include into source package.
EXTRA_DIST = $(manifest)

#
#   Tests
#

CHECK = +$(MAKE) $(AM_MAKEFLAGS) TEST_SUITE_LOG='$@.log' check

All += $(TESTS) $(check_PROGRAMS) $(check_SCRIPTS)

define test-prologue =
	$(prologue)
	echo '$(hash)!$(SHELL)' > $@
	echo '$(PROLOGUE)' >> $@
	chmod a+x $@
endef
    # Use this variable in the beginning of recipe making a test script.

check_SCRIPTS += environment.sh
environment.sh :
	$(prologue)
	printf '%s='"'"'%s'"'"'\n' $(foreach var,$(EXPORT),'$(var)' '$($(var))') > $@
	echo 'export $(EXPORT)'  >> $@
	echo 'unset $(UNEXPORT)' >> $@
INTFILES += environment.sh

#   I do not like how Automake creates ditributions:
#
#   *   It always build them. I want make-like behavior: distrubution archive should be built
#       only if files going to distribution are changed. If nothing is changed, I want make say
#       "nothing to do".
#
#   *   Distributions build from exactly the same sources are always differ.
#
#   So let me reimplement creating tarballs.

#
#   tar
#

tar := $(distdir).tar
OUTFILES += $(tar)
HELP  += tar "make uncompressed distro tarball"
PHONY += tar
All   += tar
tar : $(tar)
INTDIRS += $(distdir)
$(tar) : tar.sh $(DISTFILES)
	$(prologue)
	$(MAKE) $(AM_MAKEFLAGS) distdir
	@ : #   Specify user and group explicitly to let archive be independent on current user uid and
	@ : #   gid. Also use tar.sh wrapper to avoid dependency on (generated) files' timestamps.
	$< $(TAR) cof $@ --owner=0 --group=0 $(distdir)

#
#   tgz
#

tgz := $(tar).gz
OUTFILES += $(tgz)
HELP  += tgz "make gzip-compressed distro tarball"
PHONY += tgz
All   += tgz
tgz : $(tgz)
$(tgz) : $(tar)
	$(prologue)
	@ : # gzip by default saves (name and) timestamp of the original file, so two compressed files
	@ : # will never be the same, even if original files are exactly the same. `-n` option prevents
	@ : # saving original file (name and) timestamp.
	gzip -cn --best $< > $@

if enable_test

HELP  += check-tgz "unpack tarball and make vc All check check-dist"
PHONY += check-tgz
check-tgz : $(tgz) check-tgz.tmp/.dir
	$(prologue)
	$(srcdir)/bin/clean-dir.sh -x .dir $@.tmp
	$(TAR) xaf $(tgz) -C $@.tmp
	cd $@.tmp/$(distdir) && MAKE='$(MAKE)' ./bin/build.sh vc All check check-dist
INTDIRS += check-tgz.tmp

# Let me add two aliases for `distcheck` target,
# because I often type `checkdist` instead of `checkdist`.
HELP  += check-dist "= distcheck"
PHONY += check-dist checkdist
check-dist : distcheck
checkdist  : distcheck

endif # enable_test

#
#   rpms
#

if enable_rpm

SPATH += rpm

# Lists of .rpm files:
brpm := $(name)-$(ver)-$(rel).$(arch).rpm
# TODO: Multiple rpms support?
srpm := $(name)-$(ver)-$(rel).src.rpm
rpms := $(brpm) $(srpm)
spec := $(name).spec
OUTFILES += $(rpms)
INTFILES += $(spec)

# Convenience targets:
HELP  += brpm "make binary rpm package"
HELP  += srpm "make source rpm package"
HELP  += rpms "make both binary and source rpm packages"
PHONY += brpm srpm rpms rpm
All   += rpms
brpm : $(brpm)
srpm : $(srpm)
rpms : $(rpms)
rpm  : rpms

rpmbuild =                                       \
    $(RPMBUILD)                                  \
        -D '_topdir $(abs_builddir)/rpmbuild-$@' \
        -D '_specdir $(abs_builddir)'            \
        -D '_sourcedir $(abs_builddir)'          \
        $(null)
    # `rpmbuild` does not like relative paths, so use absolute ones.

# Binary rpm rule:
$(brpm) : $(spec) $(tgz)
	$(prologue)
	$(rpmbuild) -bb $<
	cp rpmbuild-$@/RPMS/$(arch)/$@ $@
INTDIRS += rpmbuild-$(brpm)

# Source rpm rule:
$(srpm) : $(spec) $(tgz)
	$(prologue)
	$(rpmbuild) -bs $<
	cp rpmbuild-$@/SRPMS/$@ $@
INTDIRS += rpmbuild-$(srpm)

endif # enable_rpm

#
#   rpm tests
#

if enable_rpm
if enable_test

# List of .rpmbuild.test files:
rpmbuild_tests := $(srpm:=.rpmbuild.test)
INTFILES       += $(rpmbuild_tests)

$(rpmbuild_tests:.rpmbuild.test=.rpmbuild.log) : %.rpmbuild.log : %
    #   Before running an rpmbuil test, rpm must be ready.

# .rpmbuild.test rule:
$(rpmbuild_tests) :
	$(test-prologue)
	echo '$$SRCDIR/bin/check-srpm-rpmbuild.sh $(@:.rpmbuild.test=)' >> $@
INTDIRS += check-srpm-rpmbuild.sh.tmp

# Convenience target:
HELP  += check-srpm-build "check source rpm with rpmbuild"
PHONY += check-srpm-build
check-srpm-build : $(srpm)
	$(prologue)
	$(CHECK) TESTS='$(rpmbuild_tests)'

# List of .rpmlint.test files:
rpmlint_tests := $(rpms:=.rpmlint.test)
INTFILES      += $(rpmlint_tests)

$(rpmlint_tests:.rpmlint.test=.rpmlint.log) : %.rpmlint.log : %
    #   Before running rpmlint test, rpm must be ready.

# .rpmlint.test rule:
$(rpmlint_tests) :
	@ # There is a problem here: File uploaded to Source Forge may not be immediatelly available,
	@ # so rpmlint test may fail for a few minutes.
	$(test-prologue)
	echo '$$SRCDIR/bin/check-rpm-rpmlint.sh $(@:.rpmlint.test=)' >> $@
INTDIRS += check-rpm-rpmlint.sh.tmp

# Convenience target:
HELP  += check-rpms-lint "check rpm packages with rpmlint"
PHONY += check-rpms-lint
check-rpms-lint : $(rpms)
	$(prologue)
	$(CHECK) TESTS='$(rpmlint_tests)'

# Convenience target:
HELP  += check-rpms "check rpm packages"
PHONY += check-rpms
check-rpms : check-srpm-build check-rpms-lint
	$(prologue)
	$(CHECK) TESTS='$(rpmbuild_tests) $(rpmlint_tests)'

# Convenience target alias:
PHONY += check-rpm
check-rpm : check-rpms

endif # enable_test
endif # enable_rpm

#
#   General rules.
#

% : %.in subst.sed
	$(prologue)
	@: # Let's do a special trick to have correct "file:" comment in both input and output files.
	sed \
		-e 's|[@]\s\+file:\s\+\(\S\+\)\s\+[@]|file: $@ (made from: \1)|g' \
		-f subst.sed $< > $@
	@: # TODO: Check sed in configure.

INTFILES += subst.sed
subst.sed :
	$(prologue)
	{ \
		echo 's|[@]name[@]|$(name)|g'; \
		echo 's|[@]ver[@]|$(ver)|g'; \
		echo 's|[@]rel[@]|$(rel)|g'; \
		echo 's|[@]bindir[@]|$(bindir)|g'; \
		echo 's|[@]sbindir[@]|$(sbindir)|g'; \
		echo 's|[@]srcdir[@]|$(srcdir)|g'; \
		echo 's|[@]sysconfdir[@]|$(sysconfdir)|g'; \
		echo 's|[@]PACKAGE_BRPM[@]|$(brpm)|g'; \
		echo 's|[@]PACKAGE_BUGREPORT[@]|@PACKAGE_BUGREPORT@|g'; \
		echo 's|[@]PACKAGE_COPR_PKG[@]|@PACKAGE_COPR_PKG@|g'; \
		echo 's|[@]PACKAGE_COPR_PRJ[@]|@PACKAGE_COPR_PRJ@|g'; \
		echo 's|[@]PACKAGE_COPR_REPO[@]|@PACKAGE_COPR_REPO@|g'; \
		echo 's|[@]PACKAGE_COPYRIGHT_HOLDER[@]|@PACKAGE_COPYRIGHT_HOLDER@|g'; \
		echo 's|[@]PACKAGE_COPYRIGHT_YEAR[@]|@PACKAGE_COPYRIGHT_YEAR@|g'; \
		echo 's|[@]PACKAGE_EMAIL[@]|@PACKAGE_EMAIL@|g'; \
		echo 's|[@]PACKAGE_LICENSE[@]|@PACKAGE_LICENSE@|g'; \
		echo 's|[@]PACKAGE_LICENSE_URL[@]|@PACKAGE_LICENSE_URL@|g'; \
		echo 's|[@]PACKAGE_NAME[@]|@PACKAGE_NAME@|g'; \
		echo 's|[@]PACKAGE_NVR[@]|$(name)-$(ver)-$(rel)|g'; \
		echo 's|[@]PACKAGE_RELEASE[@]|@PACKAGE_RELEASE@|g'; \
		echo 's|[@]PACKAGE_SF_FILES[@]|@PACKAGE_SF_FILES@|g'; \
		echo 's|[@]PACKAGE_SF_FORUMS[@]|@PACKAGE_SF_FORUMS@|g'; \
		echo 's|[@]PACKAGE_SF_REVIEWS[@]|@PACKAGE_SF_REVIEWS@|g'; \
		echo 's|[@]PACKAGE_SF_SUMMARY[@]|@PACKAGE_SF_SUMMARY@|g'; \
		echo 's|[@]PACKAGE_SF_TICKETS[@]|@PACKAGE_SF_TICKETS@|g'; \
		echo 's|[@]PACKAGE_SRPM[@]|$(srpm)|g'; \
		echo 's|[@]PACKAGE_STRING[@]|@PACKAGE_STRING@|g'; \
		echo 's|[@]PACKAGE_TARNAME[@]|@PACKAGE_TARNAME@|g'; \
		echo 's|[@]PACKAGE_TGZ[@]|$(tgz)|g'; \
		echo 's|[@]PACKAGE_URL[@]|@PACKAGE_URL@|g'; \
		echo 's|[@]PACKAGE_VERSION[@]|@PACKAGE_VERSION@|g'; \
	} > $@

%.gz : %
	$(prologue)
	gzip $<
	@: # TODO: Check gzip in configure.

%/.dir :
	$(prologue)
	test -d $(dir $@) || mkdir $(dir $@)
	echo > $@

# end of file #
