#   ---------------------------------------------------------------------- copyright and license ---
#
#   file: src/s4lt.m4
#
#   Copyright © 2019 Van de Bugger.
#
#   This file is a part of S4lt.
#
#   S4lt is free software: you can redistribute it and/or modify it under the terms of the GNU
#   General Public License as published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   S4lt is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
#   the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License along with S4lt.  If not, see
#   <http://www.gnu.org/licenses/>.
#
#   SPDX-License-Identifier: GPL-3.0-or-later
#
#   ---------------------------------------------------------------------- copyright and license ---


#
#   Few macros I miss in m4sugar.
#

#   S4LT
#   ------------------------------------------------------------------------------------------------
#   Someone may call `S4LT` from `configure.ac` to have `s4lt.m4` copied into his `aclocal.m4`.
#
AC_DEFUN([S4LT])


#   _s4_check_n_args( MACRO, ACTUAL, EXPECTED )
#   ------------------------------------------------------------------------------------------------
#   If ACTUAL != EXPECTED, issue error message "bad number of arguments" on behalf of MACRO.
#
m4_define([_s4_check_n_args],
    [m4_cond(
        [m4_eval([$2 != $3])],
        [1],
        [m4_fatal([$1: bad number of arguments: passed $2, expected $3])])])


#   _s4_check_arg_is_int( MACRO, ARGN, ARGV )
#   ------------------------------------------------------------------------------------------------
#   If ARGV is not an integer number (sign either plus or minus is allowed), issue error message
#   "bad arg ARGN: not an int" on behalf of MACRO.
#
m4_define([_s4_check_arg_is_int],
    [m4_cond(
        [m4_bregexp([$3], [^\s*[-+]?[0-9]+\s*$])],
        [-1],
        [m4_fatal([$1: bad arg $2: not an int: $3])])])


#   s4_ht, s4_nl, s4_cr, s4_sp, s4_ws
#   ------------------------------------------------------------------------------------------------
#   Return quoted strings of whitespace characters: tab, newline, carriage return, space, and all
#   together.
#
AC_DEFUN([s4_ht], m4_format([[%c]], [9]))
AC_DEFUN([s4_nl], m4_format([[%c]], [10]))
AC_DEFUN([s4_cr], m4_format([[%c]], [13]))
AC_DEFUN([s4_sp], m4_format([[%c]], [32]))
AC_DEFUN([s4_ws], s4_ht[]s4_nl[]s4_cr[]s4_sp)


#   s4_x(STRING, N)
#   ------------------------------------------------------------------------------------------------
#   Returns quoted STRING repeated N times. Leading and trailing spaces are trimmed from N. N can
#   be signed, both plus and minus are allowed. If N <= 0, empty string returned.
#
m4_define([_s4_x], [m4_for([i], [1], [$2], [1], [[$1]])])
AC_DEFUN([s4_x],
    [_s4_check_n_args([$0], [$#], [2])m4_ignore(
    )_s4_check_arg_is_int([$0], [2], [$2])m4_ignore(
    )m4_cond(
        [m4_eval([$2 <= 0])],
        [1],
        [],
        [_s4_x($@)])])


#   s4_left(STRING, N)
#   ------------------------------------------------------------------------------------------------
#   Returns N characters from the left side of the STRING. Leading and trailing spaces are trimmed
#   from N. If N <= 0, empty string returned. Result is a quoted string.
#
m4_define([_s4_left], [m4_format([[%.*s]], [$2], [$1])])
AC_DEFUN([s4_left],
    [_s4_check_n_args([$0], [$#], [2])m4_ignore(
    )_s4_check_arg_is_int([$0], [2], [$2])m4_ignore(
    )m4_cond(
        [m4_eval([$2 <= 0])],
        [1],
        [],
        [_s4_left([$1], s4_trim([$2]))])])


#   s4_right(STRING, N)
#   ------------------------------------------------------------------------------------------------
#   Returns N characters from the right side of the STRING. Leading and trailing spaces are trimmed
#   from N. N can be signed, both plus and minus are allowed. If N <= 0, empty string returned.
#   Result is a quoted string.
#
#   Implementation notes:
#       *   m4_substr looks as straightforward solution, but it is not obvious how to get result as
#           quoted string.
#       *   I need a regexp to match any character. `.` does not match newline, so I have to use
#           `\(\s\|\S\)`.
#       *   Construct `\{N\}` does not work, so I have to repeat `\(\s\|\S\)` many times.
#       *   Also, `$` matches at the end of every line, not at the end of string, so I have to add
#           special marker at the very end.
#
m4_define([_s4_right],
    [m4_bregexp([$1 THiS iS THe veRy eND!],
        [\(]_s4_x([\(\s\|\S\)], [$2])[\) THiS iS THe veRy eND!$],
        [[\1]])])
AC_DEFUN([s4_right],
    [_s4_check_n_args([$0], [$#], [2])m4_ignore(
    )_s4_check_arg_is_int([$0], [2], [$2])m4_ignore(
    )m4_cond(
        [m4_eval([$2 <= 0])],
        [1],
        [],
        [m4_eval([$2 >= ]m4_len([$1]))],
        [1],
        [[$1]],
        [_s4_right([$1], [$2])])])


#   s4_ltrim(STRING)
#   ------------------------------------------------------------------------------------------------
#   Trims leading whitespaces from STRING. Non-leading whitespace remains intact. Returns quoted
#   string.
#
#   Implementation notes:
#       *   "^" matches to the beginning of every line.
#       *   "\`" matches to the beginning of the string/text.
#
m4_define([_s4_ltrim], [m4_bpatsubst([[$1]], [\`\(.\)\s*], [\1])])
AC_DEFUN([s4_ltrim],
    [_s4_check_n_args([$0], [$#], [1])m4_ignore(
    )_s4_ltrim([$1])])


#   s4_rtrim(STRING)
#   ------------------------------------------------------------------------------------------------
#   Trims trailing whitespaces from STRING. Non-trailing whitespace remains intact. Returns quoted
#   string.
#
#   Implementation notes:
#       *   "$" matches to the end of every line.
#       *   "\'" matches to the end of the string/text.
#
m4_define([_s4_rtrim], [m4_bpatsubst([[$1]], [\s*\(.\)\'], [\1])])
AC_DEFUN([s4_rtrim],
    [_s4_check_n_args([$0], [$#], [1])m4_ignore(
    )_s4_rtrim([$1])])


#   s4_trim(STRING)
#   ------------------------------------------------------------------------------------------------
#   Trims leading and trailing whitespaces from STRING. Inner whitespace remains intact.
#   Returns quoted string.
#
m4_define([_s4_trim], [_s4_ltrim(_s4_rtrim([$1]))])
AC_DEFUN([s4_trim],
    [_s4_check_n_args([$0], [$#], [1])m4_ignore(
    )_s4_trim([$1])])


#   s4_atrim(STRING, STRING, ...)
#   ------------------------------------------------------------------------------------------------
#   Trims leading and trailing whitespaces from each STRING. Inner whitespace remains intact.
#   Returns comma-separated list of quoted strings. s4_atrim is convenient to preprocess macro
#   arguments, e. g.:
#
#       AC_DEFUN([BC_PATH_PROGS], [AC_PATH_PROGS(s4_atrim($@))])
#
#   So you can think "a" stands for "arguments", (or "all").
#
AC_DEFUN([s4_atrim], [m4_map_args_sep([_s4_trim(], [)], [,], $@)])


#   s4_ldrop(STRING, STRING, ...)
#   ------------------------------------------------------------------------------------------------
#   Drops empty strings from the left side of the argument list. Returns comma-separated list of
#   quoted strings.
#
AC_DEFUN([s4_ldrop],
    [m4_cond(
        [$#],
        [0],
        [],
        [$#],
        [1],
        [[$1]],
        [[$1]],
        [],
        [s4_ldrop(m4_shift($@))],
        [$@])])


#   s4_rdrop(STRING, STRING, ...)
#   ------------------------------------------------------------------------------------------------
#   Drops empty strings from the right side of the argument list. Returns comma-separated list of
#   quoted strings.
#
AC_DEFUN([s4_rdrop],
    [m4_reverse(s4_ldrop(m4_reverse($@)))])


#   s4_include(FILENAME)
#   ------------------------------------------------------------------------------------------------
#   Like M4 include, but file content is included as quoted string.
#
#   Implementation comments:
#
#   1.  If the first printf fails, no reason to continue, just exit immediately. Otherwise we will
#       include unquoted content.
#   2.  If cat fails, we have to printf closing quote anyway! However, we have to remember cat
#       status (to return it later, after printing closing quote).
#   3.  If the second printf fails, there is no reason to return its status because the caller will
#       not be able to check it anyway since unclosed quote — the caller code will be quoted.
#       However, the second printf will likely be successful, because the similar first printf was
#       successful.
#
m4_define([_s4_include],
    [m4_esyscmd(
        [printf '[' || exit $?; # See comment 1.
        cat '$1'; rc=$?;        # See comment 2.
        printf ']';             # See comment 3.
        exit $rc;])])
AC_DEFUN([s4_include],
    [_s4_check_n_args([$0], [$#], [1])m4_ignore(
    )_s4_include(m4_bpatsubst([[$1]], ['], ['\\'']))m4_ignore(
    )m4_cond(
        [m4_sysval],
        [0],
        [m4_ignore([everything is ok])],
        [m4_fatal([$0: shell command exited with status ]m4_sysval)])])


# end of file #
